import 'dart:io';
import 'package:fluartx/fluartx.dart';
import 'package:fluartx/src/collections.dart';
import './annotations.dart';
import './predicates.dart';

Predicate<FileSystemEntity> _isFile = (f) => f is File;
Predicate<FileSystemEntity> _isDirectory = (f) => f is Directory;

@beta
@nullable
List<File> listFilesSync(@nullable String path) {
  return listSync(path, filter: _isFile)?.map((f) => f as File)?.toList();
}

@beta
@nullable
List<Directory> listDirectoriesSync(@nullable String path) {
  return listSync(path, filter: _isDirectory)
      ?.map((f) => f as Directory)
      ?.toList();
}

@beta
@nullable
List<FileSystemEntity> listSync(@nullable String path,
    {@nullable Predicate<FileSystemEntity> filter}) {
  var list = _directoryOrNull(path)?.listSync(recursive: false);
  if (!Collections.isNullOrEmpty(list) && filter != null) {
    return list.where(filter).toList();
  } else {
    return list;
  }
}

@beta
@nullable
Directory _directoryOrNull(String path) {
  if (path.isNotEmpty) {
    var d = Directory(path);
    return d.existsSync() ? d : null;
  }
  return null;
}

@nonNull
Directory currentDirectory() => Directory.current;
